import numpy as np


def run(frame_preprocessing, frame):

    frame_copy = np.copy(frame)
    processed_frame = frame_preprocessing.run(frame_copy)

    # check validity
    if processed_frame.dtype != np.uint8:
        raise TypeError("Only uint8 images types supported")
    if len(processed_frame.shape) != 2:
        raise ValueError("Only 2D grayscale images supported")

    # find the background color by finding the most common value in the image
    bincount = np.bincount(processed_frame.flatten())
    bg_mean_color = np.argmax(bincount)

    # get region of interest (full image if no worm is found)
    where_worm = np.where(processed_frame != bg_mean_color)
    if len(where_worm[0]) == 0 or len(where_worm[1]) == 0:
        worm_roi = np.s_[0:processed_frame.shape[0], 0:processed_frame.shape[1]]
    else:
        worm_roi = np.s_[np.min(where_worm[0]):np.max(where_worm[0]), np.min(where_worm[1]):np.max(where_worm[1])]

    return processed_frame, bg_mean_color, worm_roi