import os
import h5py
from glob import glob

from wormpose.dataset.base_dataset import BaseFeaturesDataset
from wormpose.pose.worm_skeleton_utils import get_joint_indexes


class H5_Features(BaseFeaturesDataset):

    def __init__(self, dataset_path, video_names):

        try:
            features_file = glob(os.path.join(dataset_path, "*.h5"))[0]
        except:
            raise FileNotFoundError("Can't load sample data features, missing .h5 file in {}".format(dataset_path))

        with h5py.File(features_file, "r") as f:
            width = f['width'][:]
            skeletons = f['skeletons'][:]
            framerate = f.attrs['framerate']

        head_joint, mid_body_joint, tail_body_joint = get_joint_indexes(len(width[0]))

        features_dict = {'skeletons': skeletons,
                         'head_width': width[:, head_joint],
                         'midbody_width': width[:, mid_body_joint],
                         'tail_width': width[:, tail_body_joint],
                         'frame_rate': framerate}

        self._features = {video_names[0]: features_dict}

    def get_features(self, video_name):
        return self._features[video_name]

