from wormpose.dataset.features import calculate_crop_window_size
from wormpose.dataset.loader import load_dataset
from wormpose.images.real_dataset import RealDataset


class RealSimpleVisualizer(object):
    """
    Utility class to visualize the real images processed
    """
    def __init__(self, dataset_loader, dataset_path, video_name=None):
        features_dataset, self.frames_dataset, frame_preprocessing = load_dataset(dataset_loader, dataset_path)

        self.video_name = video_name if video_name is not None else self.frames_dataset.video_names()[0]
        output_image_shape = calculate_crop_window_size(features_dataset)
        self.real_dataset = RealDataset(frame_preprocessing=frame_preprocessing,
                                        output_image_shape=output_image_shape)

    def generate(self):
        with self.frames_dataset.open(self.video_name) as frames:
            for frame in frames:
                processed_frame, _ = self.real_dataset.process_frame(frame)
                yield frame, processed_frame


def main():
    import argparse
    import cv2

    parser = argparse.ArgumentParser()
    parser.add_argument('dataset_loader', type=str)
    parser.add_argument('dataset_path', type=str)
    parser.add_argument('--video_name', type=str, help="Optional video name. If not set, only visualize one video.")
    args = parser.parse_args()

    real_visualizer = RealSimpleVisualizer(args.dataset_loader, args.dataset_path, args.video_name)

    for orig, processed in real_visualizer.generate():
        cv2.imshow("orig", orig)
        cv2.imshow("processed", processed)
        cv2.waitKey()


if __name__ == "__main__":
    main()