import numpy as np

from wormpose.dataset.image_processing import frame_preprocessor


class RealDataset(object):

    def __init__(self,
                 frame_preprocessing,
                 output_image_shape):

        if output_image_shape[0] % 2 == 1 or  output_image_shape[1] % 2 == 1:
            raise NotImplementedError("Image width and height should be even numbers: {}, "
                                      "odd numbers not supported.".format(output_image_shape))

        self.output_image_shape = (output_image_shape[0], output_image_shape[1])
        self.frame_preprocessing = frame_preprocessing

    def process_frame(self, cur_frame):

        processed_frame, bg_mean_color, worm_roi = frame_preprocessor.run(self.frame_preprocessing, cur_frame)
        center = ((worm_roi[0].start + worm_roi[0].stop) // 2,
                  (worm_roi[1].start + worm_roi[1].stop) // 2)

        out_height, out_width = self.output_image_shape

        # simple case : just crop the desired shape centered around the region of interest
        if center[0] - out_height // 2 >= 0 \
                and center[0] + out_height // 2 < processed_frame.shape[0] \
                and center[1] - out_width // 2 >= 0 \
                and center[1] + out_width // 2 < processed_frame.shape[1]:
            roi_coord = np.s_[
                        int(center[0]) - out_height // 2:int(center[0]) + out_height // 2,
                        int(center[1]) - out_width // 2:int(center[1]) + out_width // 2]
            skel_offset = (roi_coord[1].start, roi_coord[0].start)
        # if we can't make a simple crop :
        # the region of interest of the worm is bigger than the result image or too much on the edge
        # let's copy what fits to a new empty image
        else:
            empty_frame = np.full(self.output_image_shape, bg_mean_color, dtype=processed_frame.dtype)
            roi_height, roi_width = worm_roi[0].stop - worm_roi[0].start, \
                                    worm_roi[1].stop - worm_roi[1].start

            y, x = max(0, (out_height - roi_height) // 2), max(0, (out_width - roi_width) // 2)
            zone_to_copy = np.s_[y:min(y + roi_height, out_height), x:min(x + roi_width, out_width)]
            if roi_height > out_height or roi_width > out_width:
                new_roi_height = zone_to_copy[0].stop - zone_to_copy[0].start
                new_roi_width = zone_to_copy[1].stop - zone_to_copy[1].start
                y, x = center[0] - new_roi_height // 2, center[1] - new_roi_width // 2
                worm_roi = np.s_[y:y + new_roi_height, x:x + new_roi_width]

            empty_frame[zone_to_copy] = processed_frame[worm_roi]
            processed_frame = empty_frame
            roi_coord = np.s_[0:self.output_image_shape[0], 0:self.output_image_shape[1]]

            skel_offset = (worm_roi[1].start - zone_to_copy[1].start, worm_roi[0].start - zone_to_copy[0].start)

        roi = processed_frame[roi_coord]

        return roi, skel_offset
