import numpy as np
from scipy.interpolate import interp1d


def interpolate_skeleton(skeleton, theta_dims):
    new_pos_dim = []
    for dim in range(skeleton.shape[1]):
        y = skeleton[:, dim]
        x = np.arange(y.size)
        if np.any(np.isnan(y)):
            new_pos_dim.append([np.nan] * (theta_dims + 1))
        else:
            # Interpolate the data using a cubic spline to "new_length" samples
            new_length = theta_dims + 1
            new_x = np.linspace(x.min(), x.max(), new_length)
            new_y = interp1d(x, y, kind='cubic')(new_x)
            new_pos_dim.append(new_y)
    new_pos = np.vstack(new_pos_dim).T
    return new_pos


def skeletons_to_angles(skeletons, theta_dims):
    new_skeletons = []
    for frame in range(skeletons.shape[0]):
        skeleton = skeletons[frame]
        new_skeletons.append(interpolate_skeleton(skeleton, theta_dims))
    new_skeletons = np.array(new_skeletons, skeletons.dtype)

    skel_x = new_skeletons[:, :, 0]
    skel_y = new_skeletons[:, :, 1]
    d_x = np.diff(skel_x, axis=1)
    d_y = np.diff(skel_y, axis=1)
    # calculate tangent angles.  atan2 uses angles from -pi to pi
    angles = np.arctan2(d_y, d_x)
    return angles.astype(np.float32)


def skeleton_to_angle(skeleton, theta_dims):
    new_skeleton = interpolate_skeleton(skeleton, theta_dims)
    skel_x = new_skeleton[:, 0]
    skel_y = new_skeleton[:, 1]
    d_x = np.diff(skel_x)
    d_y = np.diff(skel_y)
    # calculate tangent angles.  atan2 uses angles from -pi to pi
    angles = np.arctan2(d_y, d_x)
    return angles.astype(np.float32)


def get_joint_indexes(nb_skeleton_joints):

    head_joint = int(0.1 * nb_skeleton_joints)
    mid_body_joint = int(0.5 * nb_skeleton_joints)
    tail_body_joint = int(0.9 * nb_skeleton_joints)

    return head_joint, mid_body_joint, tail_body_joint